"use strict";
/**
 * @license
 * Copyright 2019 Google LLC. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =============================================================================
 */
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const tfjs_core_1 = require("@tensorflow/tfjs-core");
const expo_asset_1 = require("expo-asset");
const react_native_1 = require("react-native");
const RNFS = __importStar(require("react-native-fs"));
const platform_react_native_1 = require("./platform_react_native");
class BundleResourceHandler {
    constructor(modelJson, modelWeightsId) {
        this.modelJson = modelJson;
        this.modelWeightsId = modelWeightsId;
        if (modelJson == null || modelWeightsId == null) {
            throw new Error('Must pass the model json object and the model weights path.');
        }
        if (Array.isArray(modelWeightsId)) {
            throw new Error('Bundle resource IO handler does not currently support loading ' +
                'sharded weights');
        }
    }
    /**
     * Save model artifacts. This IO handler cannot support writing to the
     * packaged bundle at runtime and is exclusively for loading a model
     * that is already packages with the app.
     */
    async save() {
        throw new Error('Bundle resource IO handler does not support saving. ' +
            'Consider using asyncStorageIO instead');
    }
    /**
     * Load a model from local storage.
     *
     * See the documentation to `browserLocalStorage` for details on the saved
     * artifacts.
     *
     * @returns The loaded model (if loading succeeds).
     */
    async load() {
        const modelJson = this.modelJson;
        if (modelJson.weightsManifest.length > 1) {
            throw new Error('Bundle resource IO handler does not currently support loading ' +
                'sharded weights and the modelJson indicates that this model has ' +
                'sharded weights (more than one weights file).');
        }
        const weightsAsset = expo_asset_1.Asset.fromModule(this.modelWeightsId);
        if (weightsAsset.uri.match('^http')) {
            // In debug/dev mode RN will serve these assets over HTTP
            return this.loadViaHttp(weightsAsset);
        }
        else {
            // In release mode the assets will be on the file system.
            return this.loadLocalAsset(weightsAsset);
        }
    }
    async loadViaHttp(weightsAsset) {
        const modelJson = this.modelJson;
        // Load the weights
        const url = weightsAsset.uri;
        const requestInit = undefined;
        const response = await platform_react_native_1.fetch(url, requestInit, { isBinary: true });
        const weightData = await response.arrayBuffer();
        const modelArtifacts = Object.assign({}, modelJson);
        modelArtifacts.weightSpecs = modelJson.weightsManifest[0].weights;
        //@ts-ignore
        delete modelArtifacts.weightManifest;
        modelArtifacts.weightData = weightData;
        return modelArtifacts;
    }
    async loadLocalAsset(weightsAsset) {
        const modelJson = this.modelJson;
        let base64Weights;
        if (react_native_1.Platform.OS === 'android') {
            // On android we get a resource id instead of a regular path. We need
            // to load the weights from the res/raw folder using this id.
            try {
                const fileName = `${weightsAsset.uri}.${weightsAsset.type}`;
                base64Weights = await RNFS.readFileRes(fileName, 'base64');
            }
            catch (e) {
                throw new Error(`Error reading resource ${weightsAsset.uri}. Make sure the file is
            in located in the res/raw folder of the bundle`);
            }
        }
        else {
            try {
                base64Weights = await RNFS.readFile(weightsAsset.uri, 'base64');
            }
            catch (e) {
                throw new Error(`Error reading resource ${weightsAsset.uri}. Make sure the file is
            in located in the res/raw folder of the bundle`);
            }
        }
        const weightData = tfjs_core_1.util.encodeString(base64Weights, 'base64').buffer;
        const modelArtifacts = Object.assign({}, modelJson);
        modelArtifacts.weightSpecs = modelJson.weightsManifest[0].weights;
        //@ts-ignore
        delete modelArtifacts.weightManifest;
        modelArtifacts.weightData = weightData;
        return modelArtifacts;
    }
}
/**
 * Factory function for BundleResource IOHandler.
 *
 * This `IOHandler` only supports `load`. It is designed to support
 * loading models that have been statically bundled (at compile time)
 * with an app.
 *
 * ```js
 *  const modelJson = require('../path/to/model.json');
 *  const modelWeights = require('../path/to/model_weights.bin');
 *  async function bundleResourceIOExample() {
 *    const model =
 *      await tf.loadLayersModel(bundleResourceIO(modelJson, modelWeights));
 *
 *     const res = model.predict(tf.randomNormal([1, 28, 28, 1])) as tf.Tensor;
 *  }
 * ```
 *
 * @param modelJson The JSON object for the serialized model.
 * @param modelWeightsId An identifier for the model weights file. This is
 * generally a resourceId or a path to the resource in the app package.
 * This is typically obtained with a `require` statement.
 *
 * See
 * facebook.github.io/react-native/docs/images#static-non-image-resources
 * for more details on how to include static resources into your react-native
 * app including how to configure `metro` to bundle `.bin` files.
 *
 * @returns An instance of `IOHandler`
 */
/** @doc {heading: 'Models', subheading: 'IOHandlers'} */
function bundleResourceIO(modelJson, modelWeightsId) {
    if (typeof modelJson !== 'object') {
        throw new Error('modelJson must be a JavaScript object (and not a string).\n' +
            'Have you wrapped yor asset path in a require() statment?');
    }
    if (typeof modelWeightsId !== 'number') {
        throw new Error('modelWeightsID must be a number.\n' +
            'Have you wrapped yor asset path in a require() statment?');
    }
    return new BundleResourceHandler(modelJson, modelWeightsId);
}
exports.bundleResourceIO = bundleResourceIO;
//# sourceMappingURL=bundle_resource_io.js.map